<?php
include_once(dirname(__FILE__).'/CAS/client.php');

// Configuration pour la base de données
define('TICKET_BDD_PATH', '/var/lib/phpcas/');
define('TICKET_BDD', TICKET_BDD_PATH.'ticket-cache');
define('TICKET_TABLE', 'ticket');
#FIXME : création des répertoires de bdd en postinst + droits

class EoleCASClient extends CASClient
{
    /*
     * FONCTIONS EOLE
     */

    /*
     * Constructeur de la classe
     */
    public function EoleCASClient($server_version,
        $proxy,
        $server_hostname,
        $server_port,
        $server_uri,
        $start_session = true) {
            if (!$this->isLogoutRequest() && !empty($_GET['ticket'])){
                $session_id = session_id();
                $ticket = $_GET['ticket'];
                phpCAS::LOG("La session numéro : " . $session_id . " est associée au ticket : " . $ticket);
                $this->store_session_id($ticket, $session_id);
            }
            parent::CASClient($server_version, $proxy, $server_hostname, $server_port, $server_uri, $start_session);
        }
    /*
     * Fonctions de gestion des ids de session pour la déconnexion centralisée sauce "Eole"
     */
    private function store_session_id($ticket, $session_id){
        /*
         * Stocke le lien ticket<->session_id dans une bdd pour le logout
         */
        phpCAS::traceBegin();
        phpCAS::LOG(" # Enregistrement d'un ticket et de l'id de session #");
        if (!is_dir(TICKET_BDD_PATH)){
            phpCAS::LOG("!!!! Le répertoire pour la bdd de gestion des tickets n'existe pas !!!!");
            phpCAS::traceEnd();
            return;
        }
        if (!$db = sqlite_open(TICKET_BDD)){
            phpCAS::LOG("!!!! Impossible d'ouvrir la base de données pour la gestion du logout !!!!");
            phpCAS::traceEnd();
            return;
        }
        phpCAS::LOG("  + Initialisation de la table dans la base de données");
        $sql = 'CREATE TABLE '.TICKET_TABLE.' (';
        $sql .= 'ticket VARCHAR(70) DEFAULT NULL,';
        $sql .= 'session_id VARCHAR(70) DEFAULT NULL,';
        $sql .= 'dv TIMESTAMP DEFAULT NULL';
        $sql .= ')';
        phpCAS::LOG($sql);
        // Correction du bug 652
        @sqlite_query($db, $sql);

        $now = time();
        $dv=strftime('%Y-%m-%d %H:%M:%S', $now);
        $sql = "INSERT INTO " . TICKET_TABLE ." (ticket, session_id, dv)";
        $sql .= " VALUES ('" . sqlite_escape_string($ticket) . "', '" . sqlite_escape_string($session_id) . "', '" . sqlite_escape_string($dv) . "')";
        phpCAS::LOG("  + Insertion du ticket dans la table");
        phpCAS::LOG($sql);
        if (sqlite_query($db, $sql)){
            phpCAS::LOG("Le ticket '" . $ticket . "' a été enregistré avec comme id de session '" . $session_id . "' à '" . $dv . "'.");
            phpCAS::traceEnd();
        }else{
            phpCAS::LOG("Erreur à l'enregistrement du ticket dans la base de données.");
            phpCAS::traceEnd();
        };
    }
    private function retrieve_session($ticket){
        /*
         * Renvoie l'id de session associé au ticket
         */
        phpCAS::LOG("# Récupération de la session associée au ticket : '" . $ticket ."'.");
        if (!$db = sqlite_open(TICKET_BDD)){
            phpCAS::LOG("!!!! Impossible d'ouvrir la base de données pour la gestion du logout !!!!");
            return;
        }
        $sql = "SELECT session_id from " . TICKET_TABLE . " WHERE ticket='" . sqlite_escape_string($ticket) ."';";
        phpCAS::LOG($sql);
        $result = sqlite_query($db, $sql);
        if ($result){
            $nb = sqlite_num_rows($result);
            if ($nb == 1){
                phpCAS::LOG(" + On a retrouvé l'id de session dans la base de donnée");
                return sqlite_fetch_single($result);
            }else{
                phpCAS::LOG(" - Impossible de retrouver l'id de session");
                exit();
            }
        }else{
            phpCAS::LOG(" - Impossible de rechercher l'id de session : erreur inconnue");
        }
    }
    private function remove_stored_session_id($ticket, $session_id){
        /*
         * Supprime l'enregistrement d'une session
         */
        if (!$db = sqlite_open(TICKET_BDD)){
            phpCAS::LOG("!!!! Impossible d'ouvrir la base de données pour la gestion du logout !!!!");
            return;
        }
        $sql = "DELETE FROM " . TICKET_TABLE . " WHERE ticket='" . sqlite_escape_string($ticket) . "' AND session_id='" . sqlite_escape_string($session_id). "';";
        if (sqlite_query($db, $sql)){
            phpCAS::LOG(" + L'id de session a été supprimé de la base de données.");
        }
    }
    private function del_session($ticket){
        /*
         * Fonction de suppression de la session associée au ticket $ticket
         */
        $session_id = $this->retrieve_session($ticket);
        phpCAS::log("  + L'id de la session associée : " . $session_id);
        $fname = "/var/lib/php5/sess_" . $session_id;
        if (file_exists($fname)){
            phpCAS::log("  + Suppression du fichier de session " . $fname);
            if (unlink($fname)){
                phpCAS::log("  => La session a bien été supprimée");
                printf("Disconnected!");
            }else{
                phpCAS::log("  - !! Erreur à la suppression de la session !! ");
            }
        }else{
           phpCAS::log("  - La session a déjà été supprimée !!");
        }
        $this->remove_stored_session_id($ticket, $session_id);
    }
    function EoleLogoutRequests($check_client=true, $allowed_clients=false){
        /*
         * Fonction gérant les requêtes de logout centralisées provenant du serveur sso
         * @check_client : should the cas server be checked
         * @allowed_clients : array of trusted clients
         */
        phpCAS::traceBegin();
        if (!$this->isLogoutRequest()) {
            phpCAS::log("Not a logout request");
            phpCAS::traceEnd();
            return;
        }
        phpCAS::log("Logout requested");
        phpCAS::log("SAML REQUEST: ".$_POST['logoutRequest']);
        if ($check_client) {
            if (!$allowed_clients) {
                $allowed_clients = array( $this->getServerHostname() );
            }
            $client_ip = $_SERVER['REMOTE_ADDR'];
            $client = gethostbyaddr($client_ip);
            phpCAS::log("Client: ".$client);
            $allowed = false;
            foreach ($allowed_clients as $allowed_client) {
                if ($client == $allowed_client) {
                    phpCAS::log("Allowed client '".$allowed_client."' matches, logout request is allowed");
                    $allowed = true;
                    break;
                } else {
                    phpCAS::log("Allowed client '".$allowed_client."' does not match");
                }
            }
            if (!$allowed) {
                phpCAS::error("Unauthorized logout request from client '".$client."'");
                printf("Unauthorized!");
                phpCAS::traceExit();
                exit();
            }
        } else {
            phpCAS::log("No access control set");
        }
        // Récupération du ticket dans la requête SAML
        preg_match("|<samlp:SessionIndex>(.*)</samlp:SessionIndex>|", $_POST['logoutRequest'], $tick, PREG_OFFSET_CAPTURE, 3);
        $wrappedSamlSessionIndex = preg_replace('|<samlp:SessionIndex>|','',$tick[0][0]);
        $ticket2logout = preg_replace('|</samlp:SessionIndex>|','',$wrappedSamlSessionIndex);
        if (! preg_match('/^[SP]T-/',$ticket2logout) ) {
            phpCAS::log("Doesn't look like a ticket." );
            exit();
        }else{
            phpCAS::log("Ticket to logout: ".$ticket2logout);
        }
        $this->del_session($ticket2logout);
        phpCAS::traceExit();
        exit();
    }

    /*
     * Fonctions de gestion des détails utilisateurs
     * set et getDetails
     */
    private function setDetails($details){
        $this->details = $details;
    }
    public function getDetails(){
        if ( empty($this->details)){
            phpCAS::error('this method (getDetails) should be used only after '.__CLASS__.'::forceAuthentication() or '.__CLASS__.'::isAuthenticated()');
        }
        return $this->details;
    }
    /*
     * Fontions de gestion du xml renvoyé par le sso eole
     * set et get CasXML
     * */
    private function setCasXML($dom){
        $this->casxml = $dom;
    }
    public function getCasXML(){
        if ( empty($this->casxml)){
            phpCAS::error('this method (getCasXML) should be used only after '.__CLASS__.'::forceAuthentication() or '.__CLASS__.'::isAuthenticated()');
        }
        return $this->casxml;
    }
    private function parseTicket($dom) {
        /*
         * fonction de parsing du ticket Cas renvoyant les infos user et details
         * format du retour à traiter:
         <cas:authenticationSuccess>
         <cas:section>
         <cle1>valeur</cle1>
         <cle2>valeur</cle2>
         </cas:section>
         </cas:authenticationSuccess>

         renvoie {section:{cle1:valeur, cle2:valeur}}
         ou
         <cas:authenticationSuccess>
         <cas:section>valeur</cas:section>
         </cas:authenticationSuccess>

         renvoie {section:valeur}
         */
        $details = array();
        $user = "undefined";
        // Pour chaque section
        foreach($dom->childNodes as $section){
            // si la section a des enfants
            if ($section->hasChildNodes()){
                $subdetails = array();
                // on traite les sous-sections
                foreach($section->childNodes as $node){
                    if ($node->localName != ""){
                        if (! isset($subdetails[$node->localName])){
                            $subdetails[$node->localName] = Array($node->nodeValue);
                        }else{
                            $subdetails[$node->localName][] = $node->nodeValue;
                        }
                    }
                }
                if (sizeof($subdetails) == 0){
                    $subdetails = $section->nodeValue;
                }
                if ($section->localName != ""){
                    $details[$section->localName] = $subdetails;
                }
            }
            // si elle est elle-meme la cle
            else{
                if ($section->localName != ""){
                    $details[$section->localName] = $section->nodeValue;
                }
            }
        }
        // on récupère le nom de l'utilisateur (inclue dans le comportement traditionnel de la lib CAS)
        if (isset($details['user'])){
            if (is_array($details['user'])){
                if (isset($details['user']['user'])){
                    if (is_array($details['user']['user'])){
                        $user = $details['user']['user'][0];
                    }else{
                        $user = $details['user']['user'];
                    }
                }else{
                    $user = $details['user'][0];
                }
            }else{
                $user = $details['user'];
            }
        }elseif (isset($details['utilisateur'])){ // cas où aucun filtre eole-sso n'a été appliqué (filtre par défaut)
            if (isset($details['utilisateur']['user'])){
                if (is_array($details['utilisateur']['user'])){
                    $user = $details['utilisateur']['user'][0];
                }else{
                    $user = $details['utilisateur']['user'];
                }
            }
        }
        return array($user, $details);
    }

    /*
     * FONCTIONS ÉCRASÉES
     */
    public function retrievePT($target_service,&$err_code,&$err_msg) {
        return parent::retrievePT($target_service,$err_code,$err_msg);
    }
    public function hasPGT() {
        return parent::hasPGT();
    }
    public function getPGT() {
        return parent::getPGT();
    }
    public function wasPreviouslyAuthenticated()
    {
        phpCAS::traceBegin();

        if ( $this->isCallbackMode() ) {
            $this->callback();
        }

        $auth = FALSE;

        if ( $this->isProxy() ) {
            // CAS proxy: username and PGT must be present
            if ( $this->isSessionAuthenticated() && !empty($_SESSION['phpCAS']['pgt']) ) {
                // authentication already done
                $this->setUser($_SESSION['phpCAS']['user']);
                if(isset($_SESSION['phpCAS']['attributes'])){
                    $this->setAttributes($_SESSION['phpCAS']['attributes']);
                }
                // EOLE
                if (isset($_SESSION['phpCAS']['details'])){
                    $this->setDetails($_SESSION['phpCAS']['details']);
                }
                else{
                    $this->setDetails('__Void__');
                }
                if (isset($_SESSION['phpCAS']['casxml'])){
                    $this->setCasXML($_SESSION['phpCAS']['casxml']);
                }
                else{
                    $this->setCasXML("__VoidXML___");
                }
                // FIN EOLE
                $this->setPGT($_SESSION['phpCAS']['pgt']);
                phpCAS::trace('user = `'.$_SESSION['phpCAS']['user'].'\', PGT = `'.$_SESSION['phpCAS']['pgt'].'\'');

                // Include the list of proxies
                if (isset($_SESSION['phpCAS']['proxies'])) {
                    $this->setProxies($_SESSION['phpCAS']['proxies']);
                    phpCAS::trace('proxies = "'.implode('", "', $_SESSION['phpCAS']['proxies']).'"');
                }

                $auth = TRUE;
            } elseif ( $this->isSessionAuthenticated() && empty($_SESSION['phpCAS']['pgt']) ) {
                // these two variables should be empty or not empty at the same time
                phpCAS::trace('username found (`'.$_SESSION['phpCAS']['user'].'\') but PGT is empty');
                // unset all tickets to enforce authentication
                unset($_SESSION['phpCAS']);
                $this->setST('');
                $this->setPT('');
            } elseif ( !$this->isSessionAuthenticated() && !empty($_SESSION['phpCAS']['pgt']) ) {
                // these two variables should be empty or not empty at the same time
                phpCAS::trace('PGT found (`'.$_SESSION['phpCAS']['pgt'].'\') but username is empty');
                // unset all tickets to enforce authentication
                unset($_SESSION['phpCAS']);
                $this->setST('');
                $this->setPT('');
            } else {
                phpCAS::trace('neither user not PGT found');
            }
        } else {
            // `simple' CAS client (not a proxy): username must be present
            if ( $this->isSessionAuthenticated() ) {
                // authentication already done
                $this->setUser($_SESSION['phpCAS']['user']);
                if(isset($_SESSION['phpCAS']['attributes'])){
                    $this->setAttributes($_SESSION['phpCAS']['attributes']);
                }
                // EOLE
                if (isset($_SESSION['phpCAS']['details'])){
                    $this->setDetails($_SESSION['phpCAS']['details']);
                }
                else{
                    $this->setDetails("__Void___");
                }
                if (isset($_SESSION['phpCAS']['casxml'])){
                    $this->setCasXML($_SESSION['phpCAS']['casxml']);
                }
                else{
                    $this->setCasXML("__VoidXML___");
                }
                // FIN EOLE

                phpCAS::trace('user = `'.$_SESSION['phpCAS']['user'].'\'');

                // Include the list of proxies
                if (isset($_SESSION['phpCAS']['proxies'])) {
                    $this->setProxies($_SESSION['phpCAS']['proxies']);
                    phpCAS::trace('proxies = "'.implode('", "', $_SESSION['phpCAS']['proxies']).'"');
                }

                $auth = TRUE;
            } else {
                phpCAS::trace('no user found');
            }
        }

        phpCAS::traceEnd($auth);
        return $auth;
    }

    public function isAuthenticated()
    {
        phpCAS::traceBegin();
        $res = FALSE;
        $validate_url = '';

        if ( $this->wasPreviouslyAuthenticated() ) {
            if($this->hasST() || $this->hasPT() || $this->hasSA()){
                // User has a additional ticket but was already authenticated
                phpCAS::trace('ticket was present and will be discarded, use renewAuthenticate()');
                header('Location: '.$this->getURL());
                phpCAS::trace( "Prepare redirect to remove ticket: ".$this->getURL() );
                phpCAS::traceExit();
                exit();
            }else{
                // the user has already (previously during the session) been
                // authenticated, nothing to be done.
                phpCAS::trace('user was already authenticated, no need to look for tickets');
                $res = TRUE;
            }
        }
        else {
            if ( $this->hasST() ) {
                // if a Service Ticket was given, validate it
                phpCAS::trace('ST `'.$this->getST().'\' is present');
                $this->validateST($validate_url,$text_response,$tree_response); // if it fails, it halts
                phpCAS::trace('ST `'.$this->getST().'\' was validated');
                if ( $this->isProxy() ) {
                    $this->validatePGT($validate_url,$text_response,$tree_response); // idem
                    phpCAS::trace('PGT `'.$this->getPGT().'\' was validated');
                    $_SESSION['phpCAS']['pgt'] = $this->getPGT();
                }
                $_SESSION['phpCAS']['user'] = $this->getUser();
                if($this->hasAttributes()){
                    $_SESSION['phpCAS']['attributes'] = $this->getAttributes();
                }
                // EOLE
                $_SESSION['phpCAS']['details'] = $this->getDetails();
                $_SESSION['phpCAS']['casxml'] = $this->getCasXML();
                // FIN EOLE
                $res = TRUE;
                $logoutTicket = $this->getST();
            }
            elseif ( $this->hasPT() ) {
                // if a Proxy Ticket was given, validate it
                phpCAS::trace('PT `'.$this->getPT().'\' is present');
                $this->validatePT($validate_url,$text_response,$tree_response); // note: if it fails, it halts
                phpCAS::trace('PT `'.$this->getPT().'\' was validated');
                if ( $this->isProxy() ) {
                    $this->validatePGT($validate_url,$text_response,$tree_response); // idem
                    phpCAS::trace('PGT `'.$this->getPGT().'\' was validated');
                    $_SESSION['phpCAS']['pgt'] = $this->getPGT();
                }
                $_SESSION['phpCAS']['user'] = $this->getUser();
                if($this->hasAttributes()){
                    $_SESSION['phpCAS']['attributes'] = $this->getAttributes();
                }
                // EOLE
                $_SESSION['phpCAS']['details'] = $this->getDetails();
                $_SESSION['phpCAS']['casxml'] = $this->getCasXML();
                // FIN EOLE
                $res = TRUE;
                $logoutTicket = $this->getPT();
            }
            elseif ( $this->hasSA() ) {
                // if we have a SAML ticket, validate it.
                phpCAS::trace('SA `'.$this->getSA().'\' is present');
                $this->validateSA($validate_url,$text_response,$tree_response); // if it fails, it halts
                phpCAS::trace('SA `'.$this->getSA().'\' was validated');
                $_SESSION['phpCAS']['user'] = $this->getUser();
                $_SESSION['phpCAS']['attributes'] = $this->getAttributes();
                $res = TRUE;
                $logoutTicket = $this->getSA();
            }
            else {
                // no ticket given, not authenticated
                phpCAS::trace('no ticket found');
            }
            if ($res) {
                // call the post-authenticate callback if registered.
                if ($this->_postAuthenticateCallbackFunction) {
                    $args = $this->_postAuthenticateCallbackArgs;
                    array_unshift($args, $logoutTicket);
                    call_user_func_array($this->_postAuthenticateCallbackFunction, $args);
                }

                // if called with a ticket parameter, we need to redirect to the app without the ticket so that CAS-ification is transparent to the browser (for later POSTS)
                // most of the checks and errors should have been made now, so we're safe for redirect without masking error messages.
                // remove the ticket as a security precaution to prevent a ticket in the HTTP_REFERRER
                if ($this->_clearTicketsFromUrl) {
                    header('Location: '.$this->getURL());
                    phpCAS::trace( "Prepare redirect to : ".$this->getURL() );
                    phpCAS::traceExit();
                    exit();
                }
            }
        }

        phpCAS::traceEnd($res);
        return $res;
    }

    public function validatePT(&$validate_url,&$text_response,&$tree_response)
    {
        /*
         * Fonction de validation du ticket PT, récupère le xml et inscrit les informations associées
         */
        phpCAS::traceBegin();
        phpCAS::trace($text_response);
        // build the URL to validate the ticket
        $validate_url = $this->getServerProxyValidateURL().'&ticket='.$this->getPT();

        if ( $this->isProxy() ) {
            // pass the callback url for CAS proxies
            $validate_url .= '&pgtUrl='.urlencode($this->getCallbackURL());
        }

        // open and read the URL
        if ( !$this->readURL($validate_url,array(),$headers,$text_response,$err_msg) ) {
            phpCAS::trace('could not open URL \''.$validate_url.'\' to validate ('.$err_msg.')');
            $this->authError('PT not validated',
            $validate_url,
            TRUE/*$no_response*/);
        }

        // create new DOMDocument object
        $dom = new DOMDocument();
        // Fix possible whitspace problems
        $dom->preserveWhiteSpace = false;
        // read the response of the CAS server into a DOMDocument object
        if ( !($dom->loadXML($text_response))) {
            // read failed
            $this->authError('PT not validated',
            $validate_url,
            FALSE/*$no_response*/,
            TRUE/*$bad_response*/,
            $text_response);
        }

        // read the root node of the XML tree
        if ( !($tree_response = $dom->documentElement) ) {
            // read failed
            $this->authError('PT not validated',
            $validate_url,
            FALSE/*$no_response*/,
            TRUE/*$bad_response*/,
            $text_response);
        }
        // insure that tag name is 'serviceResponse'
        if ( $tree_response->localName != 'serviceResponse' ) {
            // bad root node
            $this->authError('PT not validated',
            $validate_url,
            FALSE/*$no_response*/,
            TRUE/*$bad_response*/,
            $text_response);
        }
        if ( $tree_response->getElementsByTagName("authenticationSuccess")->length != 0) {
            // authentication succeded, extract the user name
            $success_elements = $tree_response->getElementsByTagName("authenticationSuccess");
            if ( $success_elements->item(0)->getElementsByTagName("user")->length == 0) {
                // no user specified => error
                $this->authError('PT not validated',
                $validate_url,
                FALSE/*$no_response*/,
                TRUE/*$bad_response*/,
                $text_response);
            }

            $this->setUser(trim($success_elements->item(0)->getElementsByTagName("user")->item(0)->nodeValue));
            $this->readExtraAttributesCas20($success_elements);

            // Store the proxies we are sitting behind for authorization checking
            if ( sizeof($arr = $success_elements->item(0)->getElementsByTagName("proxy")) > 0) {
                foreach ($arr as $proxyElem) {
                    phpCAS::trace("Storing Proxy: ".$proxyElem->nodeValue);
                    $this->_proxies[] = trim($proxyElem->nodeValue);
                }
                $_SESSION['phpCAS']['proxies'] = $this->_proxies;
            }
        // EOLE
        $user_datas_tag = $tree_response->getElementsByTagName("authenticationSuccess");
        $this->setCasXML($text_response);
        $datas = $this->parseTicket($user_datas_tag->item(0));
        $this->setUser($datas[0]);
        // Specifications Eole, à remplacer par la lecture d'attributs du client officiel ?
        $this->setDetails($datas[1]);
        // FIN EOLE

        } else if ( $tree_response->getElementsByTagName("authenticationFailure")->length != 0) {
            // authentication succeded, extract the error code and message
            $auth_fail_list = $tree_response->getElementsByTagName("authenticationFailure");
            $this->authError('PT not validated',
            $validate_url,
            FALSE/*$no_response*/,
            FALSE/*$bad_response*/,
            $text_response,
            $auth_fail_list->item(0)->getAttribute('code')/*$err_code*/,
            trim($auth_fail_list->item(0)->nodeValue)/*$err_msg*/);
        } else {
            $this->authError('PT not validated',
            $validate_url,
            FALSE/*$no_response*/,
            TRUE/*$bad_response*/,
            $text_response);
        }

        $this->renameSession($this->getPT());
        // at this step, PT has been validated and $this->_user has been set,

        phpCAS::traceEnd(TRUE);
        return TRUE;
    }
    public function validateST($validate_url,&$text_response,&$tree_response)
    {
        phpCAS::traceBegin();
        // build the URL to validate the ticket
        $validate_url = $this->getServerServiceValidateURL().'&ticket='.$this->getST();
        if ( $this->isProxy() ) {
            // pass the callback url for CAS proxies
            $validate_url .= '&pgtUrl='.urlencode($this->getCallbackURL());
        }

        // open and read the URL
        if ( !$this->readURL($validate_url,array(),$headers,$text_response,$err_msg) ) {
            phpCAS::trace('could not open URL \''.$validate_url.'\' to validate ('.$err_msg.')');
            $this->authError('ST not validated',
            $validate_url,
            TRUE/*$no_response*/);
        }

        // analyze the result depending on the version
        switch ($this->getServerVersion()) {
            case CAS_VERSION_1_0:
                if (preg_match('/^no\n/',$text_response)) {
                    phpCAS::trace('ST has not been validated');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    FALSE/*$bad_response*/,
                    $text_response);
                }
                if (!preg_match('/^yes\n/',$text_response)) {
                    phpCAS::trace('ill-formed response');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    TRUE/*$bad_response*/,
                    $text_response);
                }
                // ST has been validated, extract the user name
                $arr = preg_split('/\n/',$text_response);
                $this->setUser(trim($arr[1]));
                // EOLE
                // Specification Eole (pas d'infos user en Cas 1)
                $this->setDetails("NO INFOS");
                $this->setCasXML("NO XML");
                // FIN EOLE
                break;
            case CAS_VERSION_2_0:

                // create new DOMDocument object
                $dom = new DOMDocument();
                // Fix possible whitspace problems
                $dom->preserveWhiteSpace = false;
                // read the response of the CAS server into a DOM object
                if ( !($dom->loadXML($text_response))) {
                    phpCAS::trace('dom->loadXML() failed');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    TRUE/*$bad_response*/,
                    $text_response);
                }
                // read the root node of the XML tree
                if ( !($tree_response = $dom->documentElement) ) {
                    phpCAS::trace('documentElement() failed');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    TRUE/*$bad_response*/,
                    $text_response);
                }
                // insure that tag name is 'serviceResponse'
                if ( $tree_response->localName != 'serviceResponse' ) {
                    phpCAS::trace('bad XML root node (should be `serviceResponse\' instead of `'.$tree_response->localName.'\'');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    TRUE/*$bad_response*/,
                    $text_response);
                }

                if ( $tree_response->getElementsByTagName("authenticationSuccess")->length != 0) {
                    // authentication succeded, extract the user name
                    $success_elements = $tree_response->getElementsByTagName("authenticationSuccess");
                    if ( $success_elements->item(0)->getElementsByTagName("user")->length == 0) {
                        // no user specified => error
                        $this->authError('ST not validated',
                        $validate_url,
                        FALSE/*$no_response*/,
                        TRUE/*$bad_response*/,
                        $text_response);
                    }
                    $this->setUser(trim($success_elements->item(0)->getElementsByTagName("user")->item(0)->nodeValue));
                    $this->readExtraAttributesCas20($success_elements);
                    // EOLE
                    $user_datas_tag = $tree_response->getElementsByTagName("authenticationSuccess");
                    $datas = $this->parseTicket($user_datas_tag->item(0));
                    $this->setCasXML($text_response);
                    $this->setUser($datas[0]);
                    // Specification Eole////////
                    $this->setDetails($datas[1]);
                    // FIN EOLE
                } else if ( $tree_response->getElementsByTagName("authenticationFailure")->length != 0) {
                    phpCAS::trace('<authenticationFailure> found');
                    // authentication failed, extract the error code and message
                    $auth_fail_list = $tree_response->getElementsByTagName("authenticationFailure");
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    FALSE/*$bad_response*/,
                    $text_response,
                    $auth_fail_list->item(0)->getAttribute('code')/*$err_code*/,
                    trim($auth_fail_list->item(0)->nodeValue)/*$err_msg*/);
                } else {
                    phpCAS::trace('neither <authenticationSuccess> nor <authenticationFailure> found');
                    $this->authError('ST not validated',
                    $validate_url,
                    FALSE/*$no_response*/,
                    TRUE/*$bad_response*/,
                    $text_response);
                }
                break;
        }
        $this->renameSession($this->getST());
        // at this step, ST has been validated and $this->_user has been set,
        phpCAS::traceEnd(TRUE);
        return TRUE;
    }
}
